import {Shape} from 'lincd/lib/shapes/Shape';
import {Literal, NamedNode} from 'lincd/lib/models';
import {linkedShape} from '../package';
import {dcmitype} from '../ontologies/dcmitype';
import {dcterms} from '../ontologies/dcterms';
import {linkedProperty, literalProperty} from 'lincd/lib/utils/ShapeDecorators';

@linkedShape
export class Image extends Shape {
  static targetClass: NamedNode = dcmitype.Image;
  get title() {
    return this.getValue(dcterms.title);
  }
  set title(value: string) {
    this.overwrite(dcterms.title, new Literal(value));
  }
  get description() {
    return this.getValue(dcterms.description);
  }
  set description(value: string) {
    this.overwrite(dcterms.description, new Literal(value));
  }

  /**
   * Sets the date that the image was taken.
   * Needs to confirm ISO 8601 date/time,
   * @see pattern from: https://stackoverflow.com/a/37563868
   * @see https://www.w3.org/TR/NOTE-datetime
   */
  @literalProperty({
    path: dcterms.date,
    pattern: /^\d{4}(-\d\d(-\d\d(T\d\d:\d\d(:\d\d)?(\.\d+)?(([+-]\d\d:\d\d)|Z)?)?)?)?$/i,
  })
  get date() {
    return this.getValue(dcterms.date);
  }
  set date(value: string | Date) {
    if (value instanceof Date) {
      value = value.toISOString();
    }
    this.overwrite(dcterms.date, new Literal(value));
  }
  get format() {
    return this.getValue(dcterms.format);
  }
  set format(value: string) {
    this.overwrite(dcterms.format, new Literal(value));
  }
  get creator() {
    return this.getOne(dcterms.creator) as NamedNode;
  }
  set creator(value: NamedNode) {
    this.overwrite(dcterms.creator, value);
  }
  get publisher() {
    return this.getOne(dcterms.publisher) as NamedNode;
  }
  set publisher(value: NamedNode) {
    this.overwrite(dcterms.publisher, value);
  }
  get subjects() {
    return this.getAll(dcterms.subject);
  }
}
